import React, { PureComponent } from 'react'; /* eslint-disable-line */
import { Animated, View, TouchableOpacity, Text, RefreshControl, FlatList, Keyboard } from 'react-native'; /* eslint-disable-line */
import IconIOS from 'react-native-vector-icons/Ionicons';
import _ from 'lodash';
import Header from '../components/Header';
import InputSearch from '../components/InputSearch';
import EmptyComponent from '../components/EmptyComponent';
import Loading from '../components/Loading';

// ref this.loading
// ref this.inputSearch
// ref this.header

export default class EasyScreen extends PureComponent {
  constructor(props) {
    super(props);
    this.state = {
      data: [],
      results: []
    };
    this.config = {
      headerTitle: 'Header',
      headerColor: '#387be0',
      headerContentColor: '#fff',
    };
    this.configSearch = {
      enableSearch: true,
      icon: 'md-search',
      hiddenButton: 'right',
      color: '#fff',
      colorSearching: '#00E676'
    };
    this.refreshing = false;
    this.anim = new Animated.Value(1);
    this.status = '';
    this.searching = false;
    this.paramsFilter = ['id', 'name'];
    this.configHeader = {
      iconLeft: '',
      actionLeft: ''
    };
  }

  componentDidMount() {
    this.loadData();
  }

  loadData() {
    this.onFilter('');
  }

  onFilter(text) {
    const { data } = this.state;
    let results = [];
    if (!_.isEmpty(data) && text) {
      results = _.filter(data, (item) => {
        let condition = false;
        _.each(this.paramsFilter, (key) => {
          if (item[key]) {
            if (item[key].toString().indexOf(text) > -1) {
              condition = true;
            }
          }
        });
        return condition;
      });
    } else {
      results = data;
    }
    this.setState({
      results,
    }, () => {
      this.loading && this.loading.onClose();
    });
  }

  onScroll = (event) => {
    const currentOffset = event.nativeEvent.contentOffset.y;
    const direction = currentOffset > this.offset ? 'down' : 'up';
    if (currentOffset > this.offset && (currentOffset - this.offset > 3) && this.status !== direction) {
      this.status = direction;
      this.onHideHeader();
    }
    if (currentOffset < this.offset && (this.offset - currentOffset > 3) && this.status !== direction && !this.searching) {
      this.status = direction;
      this.onShowHeader();
    }
    this.offset = currentOffset;
  }

  onShowHeader() {
    Animated.timing(this.anim, { toValue: 1 }).start();
  }

  onHideHeader() {
    Animated.timing(this.anim, { toValue: 0 }).start();
  }

  onRefresh() {
    this.loading.onOpen();
    this.inputSearch && this.inputSearch.onClearTextInput();
    this.loadData();
  }

  onChangeText(text) {
    this.onFilter(text);
  }

  renderHeader() {
    const { headerTitle, headerColor, headerContentColor } = this.config;
    const { enableSearch, icon, color, colorSearching } = this.configSearch;
    const { iconLeft, actionLeft } = this.configHeader;
    return (
      <Header
        ref={c => { this.header = c; }}
        title={headerTitle}
        iconLeft={iconLeft}
        actionLeft={actionLeft}
        navigation={this.props.navigation}
        backgroundColor={headerColor}
        color={headerContentColor}
        componentRight={enableSearch ? (
          <TouchableOpacity
            onPress={() => { this.onHideHeader(); this.status = 'down'; }}
            style={{ height: 50, width: 50, alignItems: 'center', justifyContent: 'center', backgroundColor: 'transparent' }}>
            <IconIOS
              size={30}
              name={icon}
              style={{ color: this.searching ? colorSearching : color }}
            />
          </TouchableOpacity>
        ) : <View />} />
    );
  }

  renderInput() {
    const { hiddenButton, enableSearch } = this.configSearch;
    if (!enableSearch) {
      return null;
    }
    return (
      <InputSearch
        ref={c => { this.inputSearch = c; }}
        onChangeText={(text) => {
          text ? this.searching = true : this.searching = false;
          this.onChangeText(text);
        }}
        componentLeft={hiddenButton === 'left' ?
          <TouchableOpacity
            onPress={() => {
              this.onShowHeader();
              Keyboard.dismiss();
              this.status = 'up';
            }}
            style={{ padding: 10, alignItems: 'center', justifyContent: 'center' }}>
            <Text style={{ fontWeight: 'bold' }}>Hide</Text>
          </TouchableOpacity> : null}
        componentRight={hiddenButton === 'right' ?
          <TouchableOpacity
            onPress={() => {
              this.onShowHeader();
              Keyboard.dismiss();
              this.status = 'up';
            }}
            style={{ padding: 10, alignItems: 'center', justifyContent: 'center' }}>
            <Text style={{ fontWeight: 'bold' }}>Hide</Text>
          </TouchableOpacity> : null}
      />
    );
  }

  renderItem(item, index) {
    const { name } = item;
    return (
      <TouchableOpacity
        key={index}
        style={{
          height: 50,
          flex: 1,
          alignItems: 'center',
          justifyContent: 'center',
          backgroundColor: index % 2 ? '#d0d0d0' : '#fff'
        }}>
        <Text>{`${index} - ${name}`}</Text>
      </TouchableOpacity>
    );
  }

  renderBody() {
    const { results } = this.state;
    let legacyImplementation = true;
    if (_.isEmpty(results)) {
      legacyImplementation = false;
    }
    return (
      <FlatList
        enableEmptySections
        showsVerticalScrollIndicator={false}
        legacyImplementation={legacyImplementation}
        keyboardShouldPersistTaps={'always'}
        data={results}
        keyExtractor={(item) => item.id}
        renderItem={({ item, index }) => this.renderItem(item, index)}
        ListEmptyComponent={() =>
          (<View style={{ flex: 1, height: 800 }}>
            <EmptyComponent />
          </View>)
        }
      />
    );
  }

  renderOtherComponent() {
    return null;
  }

  render() {
    const { enableSearch } = this.configSearch;
    return (
      <View style={{ flex: 1 }}>
        <View style={{ height: 25, width: '100%', backgroundColor: '#000' }} />
        {enableSearch ? (<View style={{ height: 60, width: '100%', backgroundColor: 'transparent' }} />) : null}
        <View style={{ flex: 1 }}>
          <Animated.ScrollView
            showsVerticalScrollIndicator={false}
            style={{ flex: 1, backgroundColor: 'transparent' }}
            scrollEventThrottle={1}
            onScroll={this.onScroll}
            refreshControl={
              <RefreshControl
                refreshing={this.refreshing}
                onRefresh={() => this.onRefresh()}
              />
            }
          >
            <View style={{ flex: 1 }}>
              {this.renderBody()}
            </View>
          </Animated.ScrollView>
          {this.renderOtherComponent()}
          <Loading
            ref={c => { this.loading = c; }}
            disable
            backgroundColor="#fff" />
        </View>
        <Animated.View
          style={[{
            position: 'absolute',
            top: 0,
            left: 0,
            right: 0,
            backgroundColor: 'transparent',
            overflow: 'hidden',
            borderColor: '#ddd',
            shadowColor: '#000',
            shadowOffset: { width: 0, height: 3 },
            shadowOpacity: 0.8,
            shadowRadius: 3,
            elevation: 3,
          }, {
            transform: [{
              translateY: this.anim.interpolate({
                inputRange: [0, 1],
                outputRange: [25, 0]
              })
            }]
          }]}
        >
          {this.renderInput()}
        </Animated.View>
        <Animated.View
          style={[{
            position: 'absolute',
            top: 0,
            left: 0,
            right: 0,
            backgroundColor: 'transparent',
            overflow: 'hidden',
            height: 75,
            borderColor: '#ddd',
            shadowColor: '#000',
            shadowOffset: { width: 0, height: 3 },
            shadowOpacity: 0.8,
            shadowRadius: 3,
            elevation: 3,
          }, {
            transform: [{
              translateY: this.anim.interpolate({
                inputRange: [0, 1],
                outputRange: [-75, 0]
              })
            }]
          }]}
        >
          {this.renderHeader()}
        </Animated.View>
      </View>
    );
  }
}
